@@ -44,7 +44,7 @@ def chain(rw_context):
     }
     rw_context.task = {
         'scopes': ['project:releng:signing:cert:nightly-signing', 'ignoreme'],
-        'dependencies': [],
+        'dependencies': ['decision_task_id'],
         'provisionerId': rw_context.config['provisioner_id'],
         'schedulerId': 'schedulerId',
         'workerType': rw_context.config['worker_type'],
@@ -67,6 +67,7 @@ def chain(rw_context):
 def build_link(chain):
     link = cotverify.LinkOfTrust(chain.context, 'build', 'build_task_id')
     link.cot = {
+        'taskId': 'build_task_id',
         'environment': {
             'imageArtifactHash': "sha256:built_docker_image_sha",
         },
@@ -77,7 +78,7 @@ def build_link(chain):
         'provisionerId': 'provisioner',
         'workerType': 'workerType',
         'scopes': [],
-        'dependencies': [],
+        'dependencies': ['some_task_id'],
         'metadata': {
             'source': 'https://hg.mozilla.org/mozilla-central',
         },
@@ -105,6 +106,7 @@ def build_link(chain):
                     'docker-image': 'docker_image_task_id',
                 },
             },
+            'parent': 'decision_task_id',
         },
     }
     yield link
@@ -114,6 +116,7 @@ def build_link(chain):
 def decision_link(chain):
     link = cotverify.LinkOfTrust(chain.context, 'decision', 'decision_task_id')
     link.cot = {
+        'taskId': 'decision_task_id',
         'environment': {
             'imageHash': "sha256:decision_image_sha",
         },
@@ -135,10 +138,71 @@ def decision_link(chain):
     yield link
 
 
+@pytest.yield_fixture(scope='function')
+def action_link(chain):
+    link = cotverify.LinkOfTrust(chain.context, 'action', 'action_task_id')
+    link.cot = {
+        'taskId': 'action_task_id',
+        'environment': {
+            'imageHash': "sha256:decision_image_sha",
+        },
+    }
+    link.task = {
+        'taskGroupId': 'decision_task_id',
+        'schedulerId': 'scheduler_id',
+        'provisionerId': 'provisioner_id',
+        'workerType': 'workerType',
+        'scopes': [],
+        'metadata': {
+            'source': 'https://hg.mozilla.org/mozilla-central',
+        },
+        'payload': {
+            'image': "blah",
+        },
+        'extra': {
+            'action': {},
+            'parent': 'decision_task_id',
+        },
+    }
+    yield link
+
+
+@pytest.yield_fixture(scope='function')
+def release_action_link(chain):
+    # Release action tasks look like decision tasks (self-contained graph) but
+    # are action tasks.
+    link = cotverify.LinkOfTrust(chain.context, 'decision', 'relaction_task_id')
+    link.cot = {
+        'taskId': 'relaction_task_id',
+        'environment': {
+            'imageHash': "sha256:decision_image_sha",
+        },
+    }
+    link.task = {
+        'taskGroupId': 'relaction_task_id',
+        'schedulerId': 'scheduler_id',
+        'provisionerId': 'provisioner_id',
+        'workerType': 'workerType',
+        'scopes': [],
+        'metadata': {
+            'source': 'https://hg.mozilla.org/mozilla-central',
+        },
+        'payload': {
+            'image': "blah",
+        },
+        'extra': {
+            'action': {},
+            'parent': 'decision_task_id',
+        },
+    }
+    yield link
+
+
 @pytest.yield_fixture(scope='function')
 def docker_image_link(chain):
     link = cotverify.LinkOfTrust(chain.context, 'docker-image', 'docker_image_task_id')
     link.cot = {
+        'taskId': 'docker_image_task_id',
         'artifacts': {
             'path/image': {
                 'sha256': 'built_docker_image_sha',
@@ -169,6 +233,23 @@ def docker_image_link(chain):
     yield link
 
 
+def get_cot(task_defn, task_id="task_id"):
+    return {
+        "artifacts": {
+            "path/to/artifact": {
+                "sha256": "abcd1234"
+            },
+        },
+        "chainOfTrustVersion": 1,
+        "environment": {},
+        "task": task_defn,
+        "runId": 0,
+        "taskId": task_id,
+        "workerGroup": "...",
+        "workerId": "..."
+    }
+
+
 # dependent_task_ids {{{1
 def test_dependent_task_ids(chain):
     ids = ["one", "TWO", "thr33", "vier"]
@@ -178,6 +259,20 @@ def test_dependent_task_ids(chain):
     assert sorted(chain.dependent_task_ids()) == sorted(ids)
 
 
+# get_all_links_in_chain {{{1
+@pytest.mark.asyncio
+async def test_get_all_links_in_chain(chain, decision_link, build_link):
+    # standard test
+    chain.links = [build_link, decision_link]
+    assert set(chain.get_all_links_in_chain()) == set([chain, decision_link, build_link])
+
+    # decision task test
+    chain.task_type = 'decision'
+    chain.task_id = decision_link.task_id
+    chain.links = [build_link, decision_link]
+    assert set(chain.get_all_links_in_chain()) == set([decision_link, build_link])
+
+
 # is_try {{{1
 @pytest.mark.parametrize("bools,expected", (([False, False], False), ([False, True], True)))
 def test_chain_is_try(chain, bools, expected):
@@ -198,6 +293,55 @@ def test_is_try(task):
     assert cotverify.is_try(task)
 
 
+# is_action {{{1
+@pytest.mark.parametrize("task,expected", ((
+    {
+        'payload': {
+            'env': {
+                'ACTION_CALLBACK': 'foo'
+            }
+        },
+        'extra': {
+            'action': {
+            }
+        },
+    },
+    True
+), (
+    {
+        'payload': {
+        },
+        'extra': {
+            'action': {
+            }
+        },
+    },
+    True
+), (
+    {
+        'payload': {
+            'env': {
+                'ACTION_CALLBACK': 'foo'
+            }
+        },
+    },
+    True
+), (
+    {
+        'payload': {
+            'env': {
+                'GECKO_HEAD_REPOSITORY': "https://hg.mozilla.org/try/blahblah"
+            }
+        },
+        'metadata': {},
+        'schedulerId': "x"
+    },
+    False
+)))
+def test_is_action(task, expected):
+    assert cotverify.is_action(task) == expected
+
+
 # get_link {{{1
 @pytest.mark.parametrize("ids,req,raises", ((
     ("one", "two", "three"), "one", False
@@ -229,11 +373,16 @@ def test_link_task(chain):
 
 # link.cot {{{1
 def test_link_cot(chain):
-    link = cotverify.LinkOfTrust(chain.context, 'build', "one")
-    link.cot = chain.task
-    assert link.cot == chain.task
+    link = cotverify.LinkOfTrust(chain.context, 'build', "task_id")
+    cot = get_cot(chain.task, task_id=link.task_id)
+    link.cot = cot
+    assert link.cot == cot
     with pytest.raises(CoTError):
         link.cot = {}
+    # mismatched taskId should raise
+    link2 = cotverify.LinkOfTrust(chain.context, 'build', "different_task_id")
+    with pytest.raises(CoTError):
+        link2.cot = cot
 
 
 # raise_on_errors {{{1
@@ -256,9 +405,15 @@ def test_raise_on_errors(errors, raises):
 ), (
     {'payload': {}, 'provisionerId': 'test-dummy-provisioner', 'workerType': 'test-dummy-myname', 'scopes': ["x"]},
     'scriptworker', False
+), (
+    {'payload': {'mounts': [], 'osGroups': []}, 'provisionerId': '', 'workerType': '', 'scopes': []},
+    'generic-worker', False
 ), (
     {'payload': {'image': 'x'}, 'provisionerId': 'test-dummy-provisioner', 'workerType': '', 'scopes': []},
     None, True
+), (
+    {'payload': {'image': 'x', 'osGroups': []}, 'provisionerId': '', 'workerType': '', 'scopes': []},
+    None, True
 )))
 def test_guess_worker_impl(chain, task, expected, raises):
     link = mock.MagicMock()
@@ -315,6 +470,9 @@ def test_verify_docker_image_sha(chain, build_link, decision_link, docker_image_
     chain.links = [build_link, decision_link, docker_image_link]
     for link in chain.links:
         cotverify.verify_docker_image_sha(chain, link)
+    # cover action == decision case
+    decision_link.task_type = 'action'
+    cotverify.verify_docker_image_sha(chain, decision_link)
 
 
 def test_verify_docker_image_sha_wrong_built_sha(chain, build_link, decision_link, docker_image_link):
@@ -348,13 +506,20 @@ def test_verify_docker_image_sha_bad_allowlist(chain, build_link, decision_link,
         cotverify.verify_docker_image_sha(chain, decision_link)
 
 
-# find_task_dependencies {{{1
-@pytest.mark.parametrize("task,expected", ((
-    {'taskGroupId': 'task_id', 'extra': {}, 'payload': {}},
-    {}
+# find_sorted_task_dependencies{{{1
+@pytest.mark.parametrize("task,expected,task_type", ((
+    # Make sure we don't follow other_task_id on a decision task
+    {'taskGroupId': 'other_task_id', 'extra': {}, 'payload': {}},
+    [('decision:decision', 'task_id')],
+    'decision'
 ), (
     {'taskGroupId': 'decision_task_id', 'extra': {}, 'payload': {}},
-    {'build:decision': 'decision_task_id'}
+    [('build:decision', 'decision_task_id')],
+    'build'
+), (
+    {'taskGroupId': 'decision_task_id', 'extra': {'parent': 'action_task_id'}, 'payload': {}},
+    [('build:action', 'action_task_id')],
+    'build'
 ), (
     {
         'taskGroupId': 'decision_task_id',
@@ -362,10 +527,11 @@ def test_verify_docker_image_sha_bad_allowlist(chain, build_link, decision_link,
             'chainOfTrust': {'inputs': {'docker-image': 'docker_image_task_id'}}
         },
         'payload': {},
-    }, {
-        'build:decision': 'decision_task_id',
-        'build:docker-image': 'docker_image_task_id'
-    }
+    }, [
+        ('build:decision', 'decision_task_id'),
+        ('build:docker-image', 'docker_image_task_id'),
+    ],
+    'build'
 ), (
     {
         'taskGroupId': 'decision_task_id',
@@ -381,14 +547,40 @@ def test_verify_docker_image_sha_bad_allowlist(chain, build_link, decision_link,
                 'taskType': "blah",
             }],
         },
-    }, {
-        'build:decision': 'decision_task_id',
-        'build:docker-image': 'docker_image_task_id',
-        'build:blah': 'blah_task_id',
-    }
+    }, [
+        ('build:decision', 'decision_task_id'),
+        ('build:blah', 'blah_task_id'),
+        ('build:blah', 'blah_task_id'),     # Duplicates aren't deleted
+        ('build:docker-image', 'docker_image_task_id'),
+    ],
+    'build'
+), (
+    # PushAPK-like definitions
+    {
+        'taskGroupId': 'decision_task_id',
+        'extra': {},
+        'payload': {
+            'upstreamArtifacts': [{
+                'taskId': 'platform_0_signing_task_id',
+                'taskType': 'signing',
+            }, {
+                'taskId': 'platform_1_signing_task_id',
+                'taskType': 'signing',
+            }, {
+                'taskId': 'platform_2_signing_task_id',
+                'taskType': 'signing',
+            }],
+        },
+    }, [
+        ('pushapk:decision', 'decision_task_id'),
+        ('pushapk:signing', 'platform_0_signing_task_id'),
+        ('pushapk:signing', 'platform_1_signing_task_id'),
+        ('pushapk:signing', 'platform_2_signing_task_id'),
+    ],
+    'pushapk'
 )))
-def test_find_task_dependencies(task, expected):
-    assert expected == cotverify.find_task_dependencies(task, 'build', 'task_id')
+def test_find_sorted_task_dependencies(task, expected, task_type):
+    assert expected == cotverify.find_sorted_task_dependencies(task, task_type, 'task_id')
 
 
 # build_task_dependencies {{{1
@@ -413,12 +605,12 @@ async def test_build_task_dependencies(chain, mocker, event_loop):
 
     def fake_find(task, name, _):
         if name.endswith('decision'):
-            return {}
-        return {
-            'build:decision': 'decision_task_id',
-            'build:a': 'already_exists',
-            'build:docker-image': 'die',
-        }
+            return []
+        return [
+            ('build:decision', 'decision_task_id'),
+            ('build:a', 'already_exists'),
+            ('build:docker-image', 'die'),
+        ]
 
     already_exists = mock.MagicMock()
     already_exists.task_id = 'already_exists'
@@ -427,7 +619,7 @@ async def test_build_task_dependencies(chain, mocker, event_loop):
     chain.context.queue = mock.MagicMock()
     chain.context.queue.task = fake_task
 
-    mocker.patch.object(cotverify, 'find_task_dependencies', new=fake_find)
+    mocker.patch.object(cotverify, 'find_sorted_task_dependencies', new=fake_find)
     with pytest.raises(CoTError):
         await cotverify.build_task_dependencies(chain, {}, 'too:many:colons:in:this:name:z', 'task_id')
     with pytest.raises(CoTError):
@@ -480,6 +672,7 @@ async def test_download_cot_artifact(chain, path, sha, raises, mocker, event_loo
     link.name = 'name'
     link.cot_dir = 'cot_dir'
     link.cot = {
+        'taskId': 'task_id',
         'artifacts': {
             'one': {
                 'sha256': 'sha',
@@ -577,7 +770,7 @@ def test_verify_cot_signatures_bad_sig(chain, build_link, mocker):
 def test_verify_cot_signatures(chain, build_link, mocker):
 
     def fake_body(*args, **kwargs):
-        return '{}'
+        return '{"taskId": "build_task_id"}'
 
     build_link._cot = None
     unsigned_path = os.path.join(build_link.cot_dir, 'public/chainOfTrust.json.asc')
@@ -590,7 +783,55 @@ def test_verify_cot_signatures(chain, build_link, mocker):
     cotverify.verify_cot_signatures(chain)
     assert os.path.exists(path)
     with open(path, "r") as fh:
-        assert json.load(fh) == {}
+        assert json.load(fh) == {"taskId": "build_task_id"}
+
+@pytest.mark.parametrize('payload, expected', (
+    ({}, {}),
+    (
+        {
+            'artifacts': {
+                'public/build': {
+                    'path': 'public/build',
+                    'expires': '2018-06-13T14:06:47.295419Z',
+                    'type': 'directory',
+                },
+            },
+        },
+        {
+            'artifacts': {
+                'public/build': {
+                    'path': 'public/build',
+                    'type': 'directory',
+                },
+            },
+        },
+    ), (
+        {
+            'artifacts': [{
+                'path': 'public/build',
+                'expires': '2018-06-13T14:06:47.295419Z',
+                'type': 'directory',
+                'name': 'public/build'
+            }],
+        },
+        {
+            'artifacts': [{
+                'path': 'public/build',
+                'type': 'directory',
+                'name': 'public/build'
+            }],
+        },
+    ),
+))
+def test_take_expires_out_from_artifacts_in_payload(payload, expected):
+    assert cotverify._take_expires_out_from_artifacts_in_payload(payload) == expected
+
+
+def test_wrong_take_expires_out_from_artifacts_in_payload():
+    with pytest.raises(CoTError):
+        cotverify._take_expires_out_from_artifacts_in_payload({
+            'artifacts': 0,
+        })
 
 
 # verify_link_in_task_graph {{{1
@@ -607,14 +848,20 @@ def test_verify_link_in_task_graph(chain, decision_link, build_link):
     cotverify.verify_link_in_task_graph(chain, decision_link, build_link)
 
 
-def test_verify_link_in_task_graph_fuzzy_match(chain, decision_link, build_link):
+@pytest.mark.parametrize("zero_deps", (True, False))
+def test_verify_link_in_task_graph_fuzzy_match(chain, decision_link, build_link, zero_deps):
     chain.links = [decision_link, build_link]
+    task_defn1 = deepcopy(build_link.task)
+    task_defn2 = deepcopy(chain.task)
+    if zero_deps:
+        build_link.task['dependencies'] = []
+        chain.task['dependencies'] = []
     decision_link.task_graph = {
         'bogus-task-id': {
-            'task': deepcopy(build_link.task)
+            'task': task_defn1
         },
         'bogus-task-id2': {
-            'task': deepcopy(chain.task)
+            'task': task_defn2
         }
     }
     cotverify.verify_link_in_task_graph(chain, decision_link, build_link)
@@ -666,6 +913,16 @@ def test_verify_link_in_task_graph_fuzzy_match_exception(chain, decision_link, b
         '-cx',
         'cd foo && ln -s x y && ./mach --foo taskgraph decision --bar --baz',
     ], False
+), (
+    [
+        '/home/worker/bin/run-task',
+        '--vcs-checkout=foo',
+        '--sparse-profile=taskgraph',
+        '--',
+        'bash',
+        '-cx',
+        'cd foo && ln -s x y && ./mach --foo taskgraph decision --bar --baz',
+    ], False
 ), (
     [
         '/bad/worker/bin/run-task',
@@ -703,32 +960,36 @@ def test_verify_firefox_decision_command(decision_link, command, raises):
         cotverify.verify_firefox_decision_command(decision_link)
 
 
-# verify_decision_task {{{1
+# verify_parent_task {{{1
 @pytest.mark.asyncio
-async def test_verify_decision_task(chain, decision_link, build_link, mocker):
+async def test_verify_parent_task(chain, action_link, release_action_link,
+                                  decision_link, build_link, mocker):
+    for parent_link in (action_link, release_action_link, decision_link):
+        build_link.decision_task_id = parent_link.decision_task_id
+        build_link.parent_task_id = parent_link.task_id
 
-    def task_graph(*args, **kwargs):
-        return {
-            build_link.task_id: {
-                'task': deepcopy(build_link.task)
-            },
-            chain.task_id: {
-                'task': deepcopy(chain.task)
-            },
-        }
+        def task_graph(*args, **kwargs):
+            return {
+                build_link.task_id: {
+                    'task': deepcopy(build_link.task)
+                },
+                chain.task_id: {
+                    'task': deepcopy(chain.task)
+                },
+            }
 
-    path = os.path.join(decision_link.cot_dir, "public", "task-graph.json")
-    makedirs(os.path.dirname(path))
-    touch(path)
-    chain.links = [decision_link, build_link]
-    decision_link.task['workerType'] = chain.context.config['valid_decision_worker_types'][0]
-    mocker.patch.object(cotverify, 'load_json', new=task_graph)
-    mocker.patch.object(cotverify, 'verify_firefox_decision_command', new=noop_sync)
-    await cotverify.verify_decision_task(chain, decision_link)
+        path = os.path.join(parent_link.cot_dir, "public", "task-graph.json")
+        makedirs(os.path.dirname(path))
+        touch(path)
+        chain.links = [parent_link, build_link]
+        parent_link.task['workerType'] = chain.context.config['valid_decision_worker_types'][0]
+        mocker.patch.object(cotverify, 'load_json', new=task_graph)
+        mocker.patch.object(cotverify, 'verify_firefox_decision_command', new=noop_sync)
+        await cotverify.verify_parent_task(chain, parent_link)
 
 
 @pytest.mark.asyncio
-async def test_verify_decision_task_worker_type(chain, decision_link, build_link, mocker):
+async def test_verify_parent_task_worker_type(chain, decision_link, build_link, mocker):
 
     def task_graph(*args, **kwargs):
         return {
@@ -748,15 +1009,15 @@ async def test_verify_decision_task_worker_type(chain, decision_link, build_link
     mocker.patch.object(cotverify, 'load_json', new=task_graph)
     mocker.patch.object(cotverify, 'verify_firefox_decision_command', new=noop_sync)
     with pytest.raises(CoTError):
-        await cotverify.verify_decision_task(chain, decision_link)
+        await cotverify.verify_parent_task(chain, decision_link)
 
 
 @pytest.mark.asyncio
-async def test_verify_decision_task_missing_graph(chain, decision_link, build_link, mocker):
+async def test_verify_parent_task_missing_graph(chain, decision_link, build_link, mocker):
     chain.links = [decision_link, build_link]
     decision_link.task['workerType'] = chain.context.config['valid_decision_worker_types'][0]
     with pytest.raises(CoTError):
-        await cotverify.verify_decision_task(chain, decision_link)
+        await cotverify.verify_parent_task(chain, decision_link)
 
 
 # verify_build_task {{{1
@@ -771,6 +1032,18 @@ async def test_verify_build_task_noop(chain, build_link):
     await cotverify.verify_build_task(chain, build_link)
 
 
+# verify_partials_task {{{1
+@pytest.mark.asyncio
+async def test_verify_partials_task(chain, build_link):
+    await cotverify.verify_partials_task(chain, build_link)
+
+
+@pytest.mark.asyncio
+async def test_verify_partials_task_noop(chain, build_link):
+    build_link.worker_impl = 'unknown'
+    await cotverify.verify_partials_task(chain, build_link)
+
+
 # verify_docker_image_task {{{1
 @pytest.mark.asyncio
 async def test_verify_docker_image_task(chain, docker_image_link):
@@ -796,7 +1069,7 @@ async def test_verify_docker_image_task_command(chain, docker_image_link):
 # verify_scriptworker_task {{{1
 @pytest.mark.parametrize("func", ["verify_balrog_task", "verify_beetmover_task",
                                   "verify_pushapk_task", "verify_signing_task",
-                                  "verify_scriptworker_task"])
+                                  "verify_partials_task", "verify_scriptworker_task"])
 @pytest.mark.asyncio
 async def test_verify_scriptworker_task(chain, build_link, func):
     build_link.worker_impl = 'scriptworker'
@@ -836,25 +1109,33 @@ async def test_verify_task_types(chain, decision_link, build_link, docker_image_
 # verify_docker_worker_task {{{1
 @pytest.mark.asyncio
 async def test_verify_docker_worker_task(mocker):
-    mocker.patch.object(cotverify, 'check_interactive_docker_worker', new=noop_sync)
-    mocker.patch.object(cotverify, 'verify_docker_image_sha', new=noop_sync)
-    await cotverify.verify_docker_worker_task(mock.MagicMock(), mock.MagicMock())
+    chain = mock.MagicMock()
+    link = mock.MagicMock()
+    check = mock.MagicMock()
+    mocker.patch.object(cotverify, 'check_interactive_docker_worker', new=check.method1)
+    mocker.patch.object(cotverify, 'verify_docker_image_sha', new=check.method2)
+    await cotverify.verify_docker_worker_task(chain, chain)
+    check.method1.assert_not_called()
+    check.method2.assert_not_called()
+    await cotverify.verify_docker_worker_task(chain, link)
+    check.method1.assert_called_once_with(link)
+    check.method2.assert_called_once_with(chain, link)
+
+
+# verify_generic_worker_task {{{1
+@pytest.mark.asyncio
+async def test_verify_generic_worker_task(mocker):
+    await cotverify.verify_generic_worker_task(mock.MagicMock(), mock.MagicMock())
 
 
 # verify_worker_impls {{{1
-@pytest.mark.parametrize("raises", (True, False))
 @pytest.mark.asyncio
 async def test_verify_worker_impls(chain, decision_link, build_link,
-                                   docker_image_link, raises, mocker):
+                                   docker_image_link, mocker):
     chain.links = [decision_link, build_link, docker_image_link]
     for func in cotverify.get_valid_worker_impls().values():
         mocker.patch.object(cotverify, func.__name__, new=noop_async)
-    if raises:
-        chain.worker_impl = 'docker-worker'
-        with pytest.raises(CoTError):
-            await cotverify.verify_worker_impls(chain)
-    else:
-        await cotverify.verify_worker_impls(chain)
+    await cotverify.verify_worker_impls(chain)
 
 
 # get_firefox_source_url {{{1
@@ -915,6 +1196,7 @@ async def test_trace_back_to_firefox_tree_bad_cot_product(chain):
 async def test_trace_back_to_firefox_tree_unknown_repo(chain, decision_link,
                                                        build_link, docker_image_link):
     docker_image_link.decision_task_id = 'other'
+    docker_image_link.parent_task_id = 'other'
     docker_image_link.task['metadata']['source'] = "https://hg.mozilla.org/unknown/repo"
     chain.links = [decision_link, build_link, docker_image_link]
     with pytest.raises(CoTError):
@@ -934,7 +1216,8 @@ async def test_trace_back_to_firefox_tree_docker_unknown_repo(chain, decision_li
 async def test_trace_back_to_firefox_tree_diff_repo(chain, decision_link,
                                                     build_link, docker_image_link):
     docker_image_link.decision_task_id = 'other'
-    docker_image_link.task['metadata']['source'] = "https://hg.mozilla.org/releases/mozilla-aurora"
+    docker_image_link.parent_task_id = 'other'
+    docker_image_link.task['metadata']['source'] = "https://hg.mozilla.org/releases/mozilla-beta"
     chain.links = [decision_link, build_link, docker_image_link]
     await cotverify.trace_back_to_firefox_tree(chain)
 
